@@ -51,6 +51,7 @@
 #include "wee-eval.h"
 #include "wee-hashtable.h"
 #include "wee-utf8.h"
+#include "../gui/gui-chat.h"
 #include "../gui/gui-color.h"
 #include "../plugins/plugin.h"
 
@@ -60,8 +61,6 @@
                     ((c >= 'A') && (c <= 'F')) ? c - 'A' + 10 :         \
                     c - '0')
 
-typedef uint32_t string_shared_count_t;
-
 struct t_hashtable *string_hashtable_shared = NULL;
 
 
@@ -77,7 +76,7 @@ string_strndup (const char *string, int length)
 {
     char *result;
 
-    if (!string)
+    if (!string || (length < 0))
         return NULL;
 
     if ((int)strlen (string) < length)
@@ -93,6 +92,72 @@ string_strndup (const char *string, int length)
     return result;
 }
 
+/*
+ * Cuts a string after max "length" chars, adds an optional suffix
+ * after the string if it is cut.
+ *
+ * If count_suffix == 1, the length of suffix is counted in the max length.
+ *
+ * If screen == 1, the cut is based on width of chars displayed.
+ *
+ * Note: result must be freed after use.
+ */
+
+char *
+string_cut (const char *string, int length, int count_suffix, int screen,
+            const char *cut_suffix)
+{
+    int length_result, length_cut_suffix;
+    char *result;
+    const char *ptr_string;
+
+    if (screen)
+        ptr_string = gui_chat_string_add_offset_screen (string, length);
+    else
+        ptr_string = gui_chat_string_add_offset (string, length);
+
+    if (!ptr_string || !ptr_string[0])
+    {
+        /* no cut */
+        return strdup (string);
+    }
+
+    if (cut_suffix && cut_suffix[0])
+    {
+        length_cut_suffix = strlen (cut_suffix);
+        if (count_suffix)
+        {
+            if (screen)
+                length -= utf8_strlen_screen (cut_suffix);
+            else
+                length -= length_cut_suffix;
+            if (length < 0)
+                return strdup ("");
+            if (screen)
+                ptr_string = gui_chat_string_add_offset_screen (string, length);
+            else
+                ptr_string = gui_chat_string_add_offset (string, length);
+            if (!ptr_string || !ptr_string[0])
+            {
+                /* no cut */
+                return strdup (string);
+            }
+        }
+        length_result = (ptr_string - string) + length_cut_suffix + 1;
+        result = malloc (length_result);
+        if (!result)
+            return NULL;
+        memcpy (result, string, ptr_string - string);
+        memcpy (result + (ptr_string - string), cut_suffix,
+                length_cut_suffix + 1);
+        return result;
+    }
+    else
+    {
+        return string_strndup (string, ptr_string - string);
+    }
+}
+
 /*
  * Converts string to lower case (locale independent).
  */
@@ -2912,6 +2977,12 @@ string_input_for_buffer (const char *string)
  *
  * Nested variables are supported, for example: "${var1:${var2}}".
  *
+ * Argument "list_prefix_no_replace" is a list to prevent replacements in
+ * string if beginning with one of the prefixes. For example if the list is
+ * { "if:", NULL } and string is: "${if:cond?true:false}${test${abc}}"
+ * then the "if:cond?true:false" is NOT replaced (via a recursive call) but
+ * "test${abc}" will be replaced.
+ *
  * Argument "errors" (if not NULL) is set with number of keys not found by
  * callback.
  *
@@ -2922,12 +2993,13 @@ char *
 string_replace_with_callback (const char *string,
                               const char *prefix,
                               const char *suffix,
+                              const char **list_prefix_no_replace,
                               char *(*callback)(void *data, const char *text),
                               void *callback_data,
                               int *errors)
 {
     int length_prefix, length_suffix, length, length_value, index_string;
-    int index_result, sub_count, sub_level, sub_errors;
+    int index_result, sub_count, sub_level, sub_errors, replace, i;
     char *result, *result2, *key, *key2, *value;
     const char *pos_end_name;
 
@@ -2993,15 +3065,36 @@ string_replace_with_callback (const char *string,
                 {
                     if (sub_count > 0)
                     {
-                        sub_errors = 0;
-                        key2 = string_replace_with_callback (key, prefix,
-                                                             suffix, callback,
-                                                             callback_data,
-                                                             &sub_errors);
-                        if (errors)
-                            (*errors) += sub_errors;
-                        free (key);
-                        key = key2;
+                        replace = 1;
+                        if (list_prefix_no_replace)
+                        {
+                            for (i = 0; list_prefix_no_replace[i]; i++)
+                            {
+                                if (strncmp (
+                                        key, list_prefix_no_replace[i],
+                                        strlen (list_prefix_no_replace[i])) == 0)
+                                {
+                                    replace = 0;
+                                    break;
+                                }
+                            }
+                        }
+                        if (replace)
+                        {
+                            sub_errors = 0;
+                            key2 = string_replace_with_callback (
+                                key,
+                                prefix,
+                                suffix,
+                                list_prefix_no_replace,
+                                callback,
+                                callback_data,
+                                &sub_errors);
+                            if (errors)
+                                (*errors) += sub_errors;
+                            free (key);
+                            key = key2;
+                        }
                     }
                     value = (*callback) (callback_data, (key) ? key : "");
                     if (value)
@@ -3184,6 +3277,9 @@ string_shared_free (const char *string)
 {
     string_shared_count_t *ptr_count;
 
+    if (!string)
+        return;
+
     ptr_count = (string_shared_count_t *)(string - sizeof (string_shared_count_t));
 
     (*ptr_count)--;
@@ -3192,6 +3288,204 @@ string_shared_free (const char *string)
         hashtable_remove (string_hashtable_shared, ptr_count);
 }
 
+/*
+ * Allocates a dynamic string (with a variable length).
+ *
+ * The parameter size_alloc is the initial allocated size, which must be
+ * greater than zero.
+ *
+ * Returns the pointer to the allocated string, which is initialized as empty
+ * string.
+ *
+ * The string returned can be used with following restrictions:
+ *   - changes are allowed in the string, between the first char and the final
+ *     '\0', which must not be removed nor moved,
+ *   - no other '\0' must be added in the string,
+ *   - content can be added in the string with function string_dyn_concat(),
+ *   - string can be freed with function string_dyn_free() (do NEVER call
+ *     directly free() on the string).
+ *
+ * Note: result must be freed after use with function string_dyn_free().
+ */
+
+char **
+string_dyn_alloc (int size_alloc)
+{
+    struct t_string_dyn *string_dyn;
+
+    if (size_alloc <= 0)
+        return NULL;
+
+    string_dyn = malloc (sizeof (*string_dyn));
+    if (!string_dyn)
+        return NULL;
+
+    string_dyn->string = malloc (size_alloc);
+    if (!string_dyn->string)
+    {
+        free (string_dyn);
+        return NULL;
+    }
+
+    string_dyn->string[0] = '\0';
+    string_dyn->size_alloc = size_alloc;
+    string_dyn->size = 1;
+
+    return &(string_dyn->string);
+}
+
+/*
+ * Copies "new_string" into a dynamic string and replaces its current content
+ * (adjusts its size accordingly).
+ *
+ * The string pointer (*string) is updated with the new allocated string
+ * if the string had to be extended, or the same pointer if there was enough
+ * size to copy the new string.
+ *
+ * Returns:
+ *   1: OK
+ *   0: error
+ */
+
+int
+string_dyn_copy (char **string, const char *new_string)
+{
+    struct t_string_dyn *ptr_string_dyn;
+    char *string_realloc;
+    string_dyn_size_t length_new, new_size_alloc;
+
+    if (!string || !*string)
+        return 0;
+
+    ptr_string_dyn = (struct t_string_dyn *)string;
+
+    length_new = (new_string) ? strlen (new_string) : 0;
+
+    if (length_new + 1 > ptr_string_dyn->size_alloc)
+    {
+        /* compute new size_alloc for the string + add */
+        new_size_alloc = (ptr_string_dyn->size_alloc < 2) ?
+            2 : ptr_string_dyn->size_alloc + (ptr_string_dyn->size_alloc / 2);
+        if (new_size_alloc < length_new + 1)
+            new_size_alloc = length_new + 1;
+        string_realloc = realloc (ptr_string_dyn->string, new_size_alloc);
+        if (!string_realloc)
+            return 0;
+        ptr_string_dyn->string = string_realloc;
+        *string = string_realloc;
+        ptr_string_dyn->size_alloc = new_size_alloc;
+    }
+
+    /* copy "new_string" in "string" */
+    if (new_string)
+        memmove (ptr_string_dyn->string, new_string, length_new + 1);
+    else
+        ptr_string_dyn->string[0] = '\0';
+    ptr_string_dyn->size = length_new + 1;
+
+    return 1;
+}
+
+/*
+ * Concatenates a string to a dynamic string and adjusts its size accordingly.
+ *
+ * The string pointer (*string) is updated with the new allocated string
+ * if the string had to be extended, or the same pointer if there was enough
+ * size to concatenate the new string.
+ *
+ * Returns:
+ *   1: OK
+ *   0: error
+ */
+
+int
+string_dyn_concat (char **string, const char *add)
+{
+    struct t_string_dyn *ptr_string_dyn;
+    char *string_realloc;
+    string_dyn_size_t length_add, new_size_alloc, new_size;
+
+    if (!string || !*string)
+        return 0;
+
+    if (!add || !add[0])
+        return 1;
+
+    ptr_string_dyn = (struct t_string_dyn *)string;
+
+    length_add = strlen (add);
+    new_size = ptr_string_dyn->size + length_add;
+
+    if (new_size > ptr_string_dyn->size_alloc)
+    {
+        /* compute new size_alloc for the string + add */
+        new_size_alloc = (ptr_string_dyn->size_alloc < 2) ?
+            2 : ptr_string_dyn->size_alloc + (ptr_string_dyn->size_alloc / 2);
+        if (new_size_alloc < new_size)
+            new_size_alloc = new_size;
+        string_realloc = realloc (ptr_string_dyn->string, new_size_alloc);
+        if (!string_realloc)
+        {
+            free (ptr_string_dyn->string);
+            free (ptr_string_dyn);
+            *string = NULL;
+            return 0;
+        }
+        ptr_string_dyn->string = string_realloc;
+        *string = string_realloc;
+        ptr_string_dyn->size_alloc = new_size_alloc;
+    }
+
+    /* concatenate "add" after "string" */
+    memmove (ptr_string_dyn->string + ptr_string_dyn->size - 1,
+             add,
+             length_add + 1);
+    ptr_string_dyn->size = new_size;
+
+    return 1;
+}
+
+/*
+ * Frees a dynamic string.
+ *
+ * The argument "string" is a pointer on a string returned by function
+ * string_dyn_alloc or a string pointer modified by string_dyn_concat.
+ *
+ * If free_string == 1, the string itself is freed in the structure.
+ * Otherwise the pointer (*string) remains valid after this call, and
+ * the caller must manually free the string with a call to free().
+ *
+ * Returns the pointer to the string if "free_string" is 0 (string
+ * pointer is still valid), or NULL if "free_string" is 1 (string
+ * has been freed).
+ */
+
+char *
+string_dyn_free (char **string, int free_string)
+{
+    struct t_string_dyn *ptr_string_dyn;
+    char *ptr_string;
+
+    if (!string || !*string)
+        return NULL;
+
+    ptr_string_dyn = (struct t_string_dyn *)string;
+
+    if (free_string)
+    {
+        free (ptr_string_dyn->string);
+        ptr_string = NULL;
+    }
+    else
+    {
+        ptr_string = ptr_string_dyn->string;
+    }
+
+    free (ptr_string_dyn);
+
+    return ptr_string;
+}
+
 /*
  * Frees all allocated data.
  */
