@@ -200,7 +200,11 @@ gui_completion_free_data (struct t_gui_completion *completion)
 void
 gui_completion_free (struct t_gui_completion *completion)
 {
+    if (!completion)
+        return;
+
     gui_completion_free_data (completion);
+
     free (completion);
 }
 
@@ -352,19 +356,26 @@ int
 gui_completion_nickncmp (const char *base_word, const char *nick, int max)
 {
     char *base_word2, *nick2;
-    int return_cmp;
+    int case_sensitive, return_cmp;
+
+    case_sensitive = CONFIG_BOOLEAN(config_completion_nick_case_sensitive);
 
     if (!CONFIG_STRING(config_completion_nick_ignore_chars)
         || !CONFIG_STRING(config_completion_nick_ignore_chars)[0]
         || !base_word || !nick || !base_word[0] || !nick[0]
         || gui_completion_nick_has_ignored_chars (base_word))
-        return string_strncasecmp (base_word, nick, max);
+    {
+        return (case_sensitive) ?
+            strncmp (base_word, nick, max) :
+            string_strncasecmp (base_word, nick, max);
+    }
 
     base_word2 = gui_completion_nick_strdup_ignore_chars (base_word);
     nick2 = gui_completion_nick_strdup_ignore_chars (nick);
 
-    return_cmp = string_strncasecmp (base_word2, nick2,
-                                     utf8_strlen (base_word2));
+    return_cmp = (case_sensitive) ?
+        strncmp (base_word2, nick2, utf8_strlen (base_word2)) :
+        string_strncasecmp (base_word2, nick2, utf8_strlen (base_word2));
 
     free (base_word2);
     free (nick2);
