@@ -798,6 +798,10 @@ gui_key_bind_plugin_hashtable_map_cb (void *data,
 
     if (user_data && key && value)
     {
+        /* ignore special key "__quiet" */
+        if (strcmp (key, "__quiet") == 0)
+            return;
+
         internal_code = gui_key_get_internal_code (key);
         if (internal_code)
         {
@@ -825,14 +829,19 @@ int
 gui_key_bind_plugin (const char *context, struct t_hashtable *keys)
 {
     int data[2];
+    const char *ptr_quiet;
 
     data[0] = gui_key_search_context (context);
     if (data[0] < 0)
         return 0;
 
-    gui_key_verbose = 1;
     data[1] = 0;
+
+    ptr_quiet = hashtable_get (keys, "__quiet");
+    gui_key_verbose = (ptr_quiet) ? 0 : 1;
+
     hashtable_map (keys, &gui_key_bind_plugin_hashtable_map_cb, data);
+
     gui_key_verbose = 0;
 
     return data[1];
@@ -1098,12 +1107,16 @@ gui_key_focus_command (const char *key, int context,
         {
             gui_chat_printf (NULL, _("Hashtable focus:"));
             list_keys = hashtable_get_list_keys (hashtable);
-            for (ptr_item = list_keys->items; ptr_item;
-                 ptr_item = ptr_item->next_item)
+            if (list_keys)
             {
-                gui_chat_printf (NULL, "  %s: \"%s\"",
-                                 ptr_item->data,
-                                 hashtable_get (hashtable, ptr_item->data));
+                for (ptr_item = list_keys->items; ptr_item;
+                     ptr_item = ptr_item->next_item)
+                {
+                    gui_chat_printf (NULL, "  %s: \"%s\"",
+                                     ptr_item->data,
+                                     hashtable_get (hashtable, ptr_item->data));
+                }
+                weelist_free (list_keys);
             }
         }
         if (debug)
@@ -1435,6 +1448,9 @@ gui_key_free (struct t_gui_key **keys, struct t_gui_key **last_key,
 {
     int i;
 
+    if (!key)
+        return;
+
     /* free memory */
     if (key->key)
         free (key->key);
