@@ -332,15 +332,37 @@ gui_window_tree_node_to_leaf (struct t_gui_window_tree *node,
 void
 gui_window_tree_free (struct t_gui_window_tree **tree)
 {
-    if (*tree)
+    if (!tree || !*tree)
+        return;
+
+    if ((*tree)->child1)
+        gui_window_tree_free (&((*tree)->child1));
+    if ((*tree)->child2)
+        gui_window_tree_free (&((*tree)->child2));
+
+    free (*tree);
+    *tree = NULL;
+}
+
+/*
+ * Searches a parent window which is split on the given direction
+ * ('h' or 'v').
+ */
+
+struct t_gui_window_tree *
+gui_window_tree_get_split (struct t_gui_window_tree *tree,
+                           char direction)
+{
+    if (!tree->parent_node)
+        return tree;
+
+    if (((tree->parent_node->split_horizontal) && (direction == 'h'))
+        || (!(tree->parent_node->split_horizontal) && (direction == 'v')))
     {
-        if ((*tree)->child1)
-            gui_window_tree_free (&((*tree)->child1));
-        if ((*tree)->child2)
-            gui_window_tree_free (&((*tree)->child2));
-        free (*tree);
-        *tree = NULL;
+        return tree;
     }
+
+    return gui_window_tree_get_split (tree->parent_node, direction);
 }
 
 /*
@@ -397,6 +419,9 @@ void
 gui_window_scroll_free (struct t_gui_window *window,
                         struct t_gui_window_scroll *scroll)
 {
+    if (!window || !scroll)
+        return;
+
     if (scroll->prev_scroll)
         (scroll->prev_scroll)->next_scroll = scroll->next_scroll;
     if (scroll->next_scroll)
