@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2016 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -25,7 +25,11 @@
 #define FIFO_OPTION_NAME "fifo"
 
 extern struct t_weechat_plugin *weechat_fifo_plugin;
+extern int fifo_quiet;
 extern int fifo_fd;
 extern char *fifo_filename;
 
+extern void fifo_create ();
+extern void fifo_remove ();
+
 #endif /* WEECHAT_FIFO_H */
